/* $Id: globus_replica_catalog.h,v 1.62 2001/10/05 19:22:02 meder Exp $ */

#ifndef GLOBUS_DONT_DOCUMENT_INTERNAL
/**
 * @file
 *
 * Globus Replica Catalog C API Header File
 *
 * This file declares functions, types, and macros that manage the
 * information in a replica catalog.
 * There are four categories of declarations:
 * @link globus_replica_catalog_connection
 * connection management @endlink,
 * @link globus_replica_catalog_collection
 * collection information management @endlink,
 * @link globus_replica_catalog_location
 * location information management @endlink, and
 * @link globus_replica_catalog_logicalfile
 * logical file information management @endlink.
 */
#endif /* GLOBUS_DONT_DOCUMENT_INTERNAL */

/**
 * @mainpage Globus Replica Catalog C API
 * @anchor globus_replica_catalog_api
 *
 * The Globus Replica Catalog C API provides operations to manage
 * the information in a replica catalog.  There are functions and
 * structures for @link globus_replica_catalog_connection managing
 * connections @endlink to a collection in a replica catalog, as well as
 * for managing @link globus_replica_catalog_collection collection
 * @endlink, @link globus_replica_catalog_location location @endlink,
 * and @link globus_replica_catalog_logicalfile logical file @endlink
 * information.  Each of these last three contain functions that add,
 * delete, and query their respective information.
 *
 * At a high level, a <em>replica catalog maps</em> filenames to locations.
 * Within a replica catalog, a @e collection represents an abstract set
 * of filenames.  Within a collection, a @e location associates physical
 * storage with a subset of collection filenames.  Also within a
 * collection, a @e logical @e file associates information such as size
 * with a collection filename.
 *
 * Further, a @e filename is information associated primarily with a
 * collection, and subordinately with a location or logical file.  Note
 * that collections and locations may have multiple filenames associated
 * with them, while a logical file's name is itself a filename.
 * Filenames may include paths which may be interpreted
 * relative to a physical root when associated with a location.
 * 
 * This library defines and uses the @link
 * globus_replica_catalog_attribute_set attribute @endlink and @link
 * globus_replica_catalog_entry_set entry @endlink sets from the Globus
 * Replica Catalog Sets C API to help manipulate the information we get
 * back from a replica catalog.  In terms of this API, collections,
 * locations, and logical files are @e entries, and information
 * associated with these entries, such as filenames, are @e attributes.
 *
 * The current implementation uses LDAP to talk to a replica catalog
 * server and requires an LDAPv2 C client library like the one available
 * from <a href="http://www.openldap.org/">OpenLDAP</a>.
 *
 * Any program that uses this library must include
 * "globus_replica_catalog.h".
 */

#ifndef _GLOBUS_REPLICA_CATALOG_H
#define _GLOBUS_REPLICA_CATALOG_H

#include "globus_common.h"
#include "globus_replica_catalog_sets.h"
#include "globus_error_string.h"
#include "lber.h"
#include "ldap.h"
#include <stdarg.h>
#include <string.h>

#ifdef EXTERN_C_BEGIN
EXTERN_C_BEGIN
#endif

#ifndef GLOBUS_DONT_DOCUMENT_INTERNAL

/** Replica catalog error format. */
#define GLOBUS_REPLICA_CATALOG_ERROR(base, text) \
	globus_error_put( globus_error_construct_string( \
		GLOBUS_REPLICA_CATALOG_MODULE, \
		(base), \
		"%s [%s, %s, %d]\n", \
		(text), \
		GLOBUS_REPLICA_CATALOG_MODULE->module_name, \
		__FILE__, \
		__LINE__ ) )

/* structs from globus_replica_catalog_sets.h */
struct globus_replica_catalog_attribute_set_s;
struct globus_replica_catalog_entry_set_s;

/** Default objectclass name of a collection entry. */
#define GLOBUS_REPLICA_CATALOG_COLLECTION_OBJECTCLASS "GlobusReplicaLogicalCollection"
/** RDN attribute name of a collection entry. */
#define GLOBUS_REPLICA_CATALOG_COLLECTION_RDN_TYPE "lc"

/** Default objectclass name of a location entry. */
#define GLOBUS_REPLICA_CATALOG_LOCATION_OBJECTCLASS "GlobusReplicaLocation"
/** RDN attribute name of a location entry. */
#define GLOBUS_REPLICA_CATALOG_LOCATION_RDN_TYPE "re"

/** Default objectclass name of a logical file entry. */
#define GLOBUS_REPLICA_CATALOG_LOGICALFILE_OBJECTCLASS "GlobusReplicaLogicalFile"
/** RDN attribute name of a logical file entry. */
#define GLOBUS_REPLICA_CATALOG_LOGICALFILE_RDN_TYPE "rf"

/** Default objectclass name of a logical file parent entry. */
#define GLOBUS_REPLICA_CATALOG_FILEPARENT_OBJECTCLASS "GlobusReplicaFileGroup"
/** RDN attribute of a logical file parent entry. */
#define GLOBUS_REPLICA_CATALOG_FILEPARENT_RDN "fin=LFROOT"

/** Filename attribute name. */
#define GLOBUS_REPLICA_CATALOG_FILENAME_ATTRIBUTE_TYPE "filename"

#endif /* GLOBUS_DONT_DOCUMENT_INTERNAL */

/**
 * @defgroup globus_replica_catalog_connection Connection Management
 *
 * Manage a connection to a collection in a replica catalog.
 *
 * As an example let's configure, open, then close a connection to some
 * collection information.  First, set up the connection attributes and
 * open a connection:
 *
 * @code
 * globus_replica_catalog_collection_handleattr_t handleattrs;
 * globus_replica_catalog_collection_handle_t handle;
 * globus_replica_catalog_collection_handleattr_init( &handleattrs );
 * globus_replica_catalog_collection_handleattr_set_authentication_mode(
 *         &handleattrs,
 *         GLOBUS_REPLICA_CATALOG_AUTHENTICATION_MODE_CLEARTEXT,
 *         "cn=Directory Manager,dc=my,dc=org,o=grid",
 *         "AdminPassword" );
 * globus_replica_catalog_collection_open( &handle,
 *         &handleattrs,
 *         "ldap://host.my.org/lc=coll23,rc=catalog1,dc=my,dc=org,o=Grid" );
 * @endcode
 *
 * @b Note: in the current implementation only the cleartext
 * authentication mode is available.  It must be set explicitly as it
 * is not the default.
 *
 * Now we can do something with the connection to the collection
 * information (omitted).  Finally, remember to close the connection and
 * clear the connection attributes after we're finished:
 *
 * @code
 * globus_replica_catalog_collection_close( &handle );
 * globus_replica_catalog_collection_handleattr_destroy( &handleattrs );
 * @endcode
 */

/**
 * Module Descriptor.
 *
 * Before using any replica catalog functions call
 * @code
 * 	globus_module_activate( GLOBUS_REPLICA_CATALOG_MODULE );
 * @endcode
 *
 * After using all replica catalog functions call
 * @code
 * 	globus_module_deactivate( GLOBUS_REPLICA_CATALOG_MODULE );
 * @endcode
 *
 * @ingroup globus_replica_catalog_connection
 */
#define GLOBUS_REPLICA_CATALOG_MODULE (&globus_i_replica_catalog_module)
extern globus_module_descriptor_t globus_i_replica_catalog_module;

/**
 * Connection authentication mode.
 *
 * @ingroup globus_replica_catalog_connection
 *
 * @see globus_replica_catalog_collection_handleattr_set_authentication_mode(),
 * 	globus_replica_catalog_collection_handleattr_get_authentication_mode()
 */
typedef enum globus_replica_catalog_authentication_mode_e
{
    /**
     * GSI authentication mode, the default, and not currently supported
     * by this implementation.
     */
    GLOBUS_REPLICA_CATALOG_AUTHENTICATION_MODE_SECURE,

    /**
     * Username/password authentication mode, currently supported by
     * this implementation.
     */
    GLOBUS_REPLICA_CATALOG_AUTHENTICATION_MODE_CLEARTEXT
}
globus_replica_catalog_authentication_mode_t;

/**
 * Connection attributes.
 *
 * @ingroup globus_replica_catalog_connection
 *
 * @see globus_replica_catalog_collection_handleattr_init(),
 * 	globus_replica_catalog_collection_handleattr_destroy(),
 * 	globus_replica_catalog_collection_handleattr_set_authentication_mode(),
 * 	globus_replica_catalog_collection_handleattr_get_authentication_mode()
 */
typedef struct globus_replica_catalog_collection_handleattr_s
{
#ifndef GLOBUS_DONT_DOCUMENT_INTERNAL

    /** type of authentication */
    globus_replica_catalog_authentication_mode_t authentication_mode;

    /** manager DN */
    char * username;

    /** manager password */
    char * password;

    /** mutex protecting the handleattr */
    globus_mutex_t mutex;

#endif /* GLOBUS_DONT_DOCUMENT_INTERNAL */
}
globus_replica_catalog_collection_handleattr_t;

/**
 * Initialize the connection attributes.
 *
 * Initialize a given set of connection attributes before use.  The
 * authentication mode is initially set to
 * #GLOBUS_REPLICA_CATALOG_AUTHENTICATION_MODE_SECURE.
 *
 * @ingroup globus_replica_catalog_connection
 *
 * @param handleattrs
 * 	  connection attributes @b returned, caller-allocated
 *
 * @see globus_replica_catalog_collection_handleattr_destroy()
 */
globus_result_t
globus_replica_catalog_collection_handleattr_init(
	globus_replica_catalog_collection_handleattr_t * handleattrs );

/**
 * Clear the connection attributes.
 *
 * Reset the connection attributes.  The authentication mode is reset to
 * #GLOBUS_REPLICA_CATALOG_AUTHENTICATION_MODE_SECURE.
 *
 * @ingroup globus_replica_catalog_connection
 *
 * @param handleattrs
 * 	  connection attributes
 *
 * @see globus_replica_catalog_collection_handleattr_init()
 */
globus_result_t
globus_replica_catalog_collection_handleattr_destroy(
	globus_replica_catalog_collection_handleattr_t * handleattrs );

/**
 * Copy the connection attributes.
 *
 * This function initializes the destination attributes with the
 * settings in the source attributes.
 *
 * @ingroup globus_replica_catalog_connection
 *
 * @param dest
 * 	  target connection attributes
 * @param src
 * 	  source connection attributes
 *
 */
globus_result_t
globus_replica_catalog_collection_handleattr_copy(
    globus_replica_catalog_collection_handleattr_t *    dest,
    globus_replica_catalog_collection_handleattr_t *    src);

/**
 * Set the authentication mode of the connection attributes.
 *
 * The current implementation requires the authentication mode be set to
 * #GLOBUS_REPLICA_CATALOG_AUTHENTICATION_MODE_CLEARTEXT.
 *
 * To connect to a replica catalog that allows unauthenticated queries
 * (generally not updates as well), set the authentication mode to
 * #GLOBUS_REPLICA_CATALOG_AUTHENTICATION_MODE_CLEARTEXT and pass in
 * GLOBUS_NULL for both @e username and @e password.
 *
 * @ingroup globus_replica_catalog_connection
 *
 * @param handleattrs
 * 	  connection attributes
 * @param authentication_mode
 *        type of authentication to eventually connect with
 * @param ...
 *        if @e authentication_mode is
 *        #GLOBUS_REPLICA_CATALOG_AUTHENTICATION_MODE_SECURE then there
 *        are no additional arguments, if @e authentication_mode is
 *        #GLOBUS_REPLICA_CATALOG_AUTHENTICATION_MODE_CLEARTEXT then the
 *        additional arguments are
 *        @code
 *        char * username
 *        char * password
 *        @endcode
 *        where @e username and @e password are the catalog manager DN
 *        and password
 *
 * @see globus_replica_catalog_collection_handleattr_get_authentication_mode()
 */
globus_result_t
globus_replica_catalog_collection_handleattr_set_authentication_mode(
	globus_replica_catalog_collection_handleattr_t * handleattrs,
	globus_replica_catalog_authentication_mode_t authentication_mode,
	... );

/**
 * Get the authentication mode of the connection attributes.
 * 
 * @ingroup globus_replica_catalog_connection
 *
 * @param handleattrs
 * 	  connection attributes
 * @param authentication_mode
 *        type of authentication @b returned
 * @param ...
 *        if @e authentication_mode is
 *        #GLOBUS_REPLICA_CATALOG_AUTHENTICATION_MODE_SECURE then there
 *        are no additional arguments, if @e authentication_mode is
 *        #GLOBUS_REPLICA_CATALOG_AUTHENTICATION_MODE_CLEARTEXT then the
 *        additional arguments @b returned are
 *        @code
 *        char ** username
 *        char ** password
 *        @endcode
 *        where @e username and @e password are the catalog manager DN
 *        and password and are pointers into an existing structure, so
 *        don't fool around with them
 *
 * @see globus_replica_catalog_collection_handleattr_set_authentication_mode()
 */
globus_result_t
globus_replica_catalog_collection_handleattr_get_authentication_mode(
	globus_replica_catalog_collection_handleattr_t * handleattrs,
	globus_replica_catalog_authentication_mode_t * authentication_mode,
	... );

/**
 * Collection handle.
 *
 * @ingroup globus_replica_catalog_connection
 *
 * @see globus_replica_catalog_collection_open(),
 * 	globus_replica_catalog_collection_create(),
 * 	globus_replica_catalog_collection_close(),
 * 	globus_replica_catalog_collection_delete()
 */
typedef struct globus_replica_catalog_collection_handle_s
{
#ifndef GLOBUS_DONT_DOCUMENT_INTERNAL

    /** connection attributes */
    globus_replica_catalog_collection_handleattr_t * attrs;

     /** parsed collection URL */
    LDAPURLDesc * urld;

     /** DN of the collection, copied initially from urld because it may
      * be reset */
    char * dn;

    /** LDAP connection handle */
    LDAP * ld;

    /** mutex to protect the handle */
    globus_mutex_t mutex;
 
#endif /* GLOBUS_DONT_DOCUMENT_INTERNAL */
}
globus_replica_catalog_collection_handle_t;

/**
 * Open a connection to a collection in a replica catalog.
 *
 * Initialize a given collection handle and open a
 * connection to the given URL with the given connection attributes.
 *
 * The collection must already exist.
 *
 * @ingroup globus_replica_catalog_connection
 *
 * @param handle
 * 	  handle to the collection @b returned, caller-allocated
 * @param handleattrs
 * 	  connection attributes to connect with
 * @param url
 * 	  collection LDAP URL to connect to, of the form
 * 	  @c ldap://host[:port]/dn
 *
 * @see globus_replica_catalog_collection_close(),
 * 	globus_replica_catalog_collection_delete()
 */
globus_result_t
globus_replica_catalog_collection_open(
	globus_replica_catalog_collection_handle_t * handle,
	globus_replica_catalog_collection_handleattr_t * handleattrs,
	char * url );

/**
 * List the collections in a replica catalog.
 *
 * For each collection only return attribute information for the given
 * attribute names.  The LDAP URL given to a preceding call to
 * globus_replica_catalog_collection_open() should include the base DN
 * to search from.
 *
 * Call globus_replica_catalog_entry_set_destroy() after final use of the
 * returned locations.
 *
 * @ingroup globus_replica_catalog_connection
 *
 * @param handle
 * 	  handle to the collection
 * @param attribute_names
 * 	  null-terminated array of names of the attributes to get, if
 * 	  GLOBUS_NULL then all attributes are returned
 * @param collections
 * 	  collections @b returned
 *
 * @see globus_replica_catalog_entry_set_destroy()
 */
globus_result_t
globus_replica_catalog_list_collections(
	globus_replica_catalog_collection_handle_t * handle,
	char ** attribute_names,
	struct globus_replica_catalog_entry_set_s * collections );

/**
 * List the rollback and restart information entries in replica catalog.
 *
 * @ingroup globus_replica_catalog_connection
 *
 * @param url
 *        URL of the base to begin searching from
 * @param handleattrs
 *        collection attributes for the implied connection, may be
 *        GLOBUS_NULL for default attributes
 * @param attribute_names
 * 	  null-terminated array of names of the attributes to get, if
 * 	  GLOBUS_NULL then all attributes are returned
 * @param rollback_entries
 * 	  rollback and restart information entries @b returned
 */
globus_result_t
globus_replica_catalog_list_rollback_entries(
    char *                                           url,
    globus_replica_catalog_collection_handleattr_t * handleattrs,
    char **                                          attribute_names,
    struct globus_replica_catalog_entry_set_s *      rollback_entries );

/**
 * Set the DN of the collection.
 *
 * All subsequent operations will be done with respect to the given DN.
 *
 * @ingroup globus_replica_catalog_connection
 *
 * @param handle
 * 	  handle to the collection
 * @param collection_dn
 * 	  collection DN to set
 */
globus_result_t
globus_replica_catalog_collection_set_dn(
	globus_replica_catalog_collection_handle_t * handle,
	char * collection_dn );

/**
 * Create a collection in a replica catalog.
 *
 * Initialize a given collection handle, open a connection to a replica
 * catalog and create a collection based on the given URL, connection
 * attributes, and objectclass.  Also, associate the given filenames
 * with the created collection.
 *
 * The collection must not already exist.
 *
 * @ingroup globus_replica_catalog_connection
 *
 * @param handle
 * 	  handle to the collection @b returned, caller-allocated
 * @param handleattrs
 * 	  connection attributes to connect with
 * @param url
 * 	  collection URL to connect to, of the form
 * 	  @c ldap://host[:port]/dn
 * @param filenames
 * 	  null-terminated array of filenames (which may include paths)
 * 	  to add, may be GLOBUS_NULL
 * @param objectclass
 * 	  objectclass name of the collection to be created, must be a
 * 	  subclass of #GLOBUS_REPLICA_CATALOG_COLLECTION_OBJECTCLASS, if
 * 	  GLOBUS_NULL is given then the default
 * 	  #GLOBUS_REPLICA_CATALOG_COLLECTION_OBJECTCLASS is used
 *
 * @see globus_replica_catalog_collection_close(),
 * 	globus_replica_catalog_collection_delete()
 */
globus_result_t
globus_replica_catalog_collection_create(
	globus_replica_catalog_collection_handle_t * handle,
	globus_replica_catalog_collection_handleattr_t * handleattrs,
	char * url,
	char ** filenames,
	char * objectclass );

/**
 * Close a connection to a collection.
 *
 * @ingroup globus_replica_catalog_connection
 *
 * @param handle
 * 	  handle to the collection
 *
 * @see globus_replica_catalog_collection_open(),
 * 	globus_replica_catalog_collection_create()
 */
globus_result_t
globus_replica_catalog_collection_close(
	globus_replica_catalog_collection_handle_t * handle );

/**
 * Delete a collection and close the connection.
 *
 * There must be no locations in the collection. This function does
 * not check for the presence of logical files in the collection.
 *
 * @ingroup globus_replica_catalog_connection
 *
 * @param handle
 * 	  handle to the collection
 *
 * @see globus_replica_catalog_collection_open(),
 * 	globus_replica_catalog_collection_create()
 */
globus_result_t
globus_replica_catalog_collection_delete(
	globus_replica_catalog_collection_handle_t * handle );

/**
 * @defgroup globus_replica_catalog_collection Collection Information Management
 *
 * Manage collection information in a replica catalog.
 *
 * These functions manage the information of an existing collection
 * represented by a given collection handle.  These functions use a
 * collection handle that has been initialized by either
 * globus_replica_catalog_collection_open() or
 * globus_replica_catalog_collection_create().
 *
 * At a high level, a @e collection represents an abstract set of
 * filenames.  The filename information associated with a collection
 * needn't necesarily have a physical counterpart.  To associate a
 * filename in a collection to physical storage see the @link
 * globus_replica_catalog_location location information management
 * @endlink functions.  To associate extra information such as size with
 * a filename in a collection see the @link
 * globus_replica_catalog_logicalfile logical file information
 * management @endlink functions.
 *
 * \todo Include a usage example.
 */

/**
 * Add filenames to a collection.
 *
 * If some of the given filenames already exist in the collection, and
 * @e add is @c GLOBUS_TRUE, then add those that don't.  Otherwise, fail if
 * some of the given filenames already exist and @e add is GLOBUS_FALSE.
 *
 * @ingroup globus_replica_catalog_collection
 *
 * @param handle
 * 	  handle to the collection
 * @param filenames
 * 	  null-terminated array of filenames (which may include paths)
 * 	  to add
 * @param add
 * 	  whether to add given filenames that don't exist in the
 * 	  collection even if there are some given filenames that do exist
 * 
 * @see globus_replica_catalog_collection_delete_filenames(),
 * 	globus_replica_catalog_collection_list_filenames()
 */
globus_result_t
globus_replica_catalog_collection_add_filenames(
	globus_replica_catalog_collection_handle_t * handle,
	char ** filenames,
	globus_bool_t add );

/**
 * Add attribute information (other than filenames) to a collection.
 * Does not replace existing attribute values.
 *
 * Note that the arbitrary information associated with a collection may
 * apply either to the collection itself or to all the filenames in the
 * collection depending on the meaning the caller wants to associate
 * with a particular attribute.
 *
 * @ingroup globus_replica_catalog_collection
 *
 * @param handle
 * 	  handle to the collection
 * @param attribute_name
 * 	  name of the attribute to add
 * @param attribute_value
 * 	  value of the attribute to add
 *
 * @see globus_replica_catalog_collection_delete_attribute(),
 * 	globus_replica_catalog_collection_list_attributes()
 */
globus_result_t
globus_replica_catalog_collection_add_attribute(
	globus_replica_catalog_collection_handle_t * handle,
	char * attribute_name,
	char * attribute_value );

/**
 * Delete filenames from a collection.  If one or more of the given
 * filenames is not associated with the collection then no deletions are
 * done.
 *
 * There must be no correspondingly named logical files or corresponding
 * filenames in any of the locations in the collection.
 *
 * @ingroup globus_replica_catalog_collection
 *
 * @param handle
 * 	  handle to the collection
 * @param filenames
 * 	  null-terminated array of filenames (which may include paths)
 * 	  to delete
 *
 * @see globus_replica_catalog_collection_add_filenames(),
 * 	globus_replica_catalog_collection_list_filenames()
 */
globus_result_t
globus_replica_catalog_collection_delete_filenames(
	globus_replica_catalog_collection_handle_t * handle,
	char ** filenames );

/**
 * Delete attribute information other that filenames from a collection.
 *
 * @ingroup globus_replica_catalog_collection
 *
 * @param handle
 * 	  handle to the collection
 * @param attribute_name
 * 	  name of the attribute to delete
 * @param attribute_value
 * 	  value of the attribute to delete, if GLOBUS_NULL then delete
 * 	  all the values associated with @e name
 *
 * @see globus_replica_catalog_collection_add_attribute(),
 * 	globus_replica_catalog_collection_list_attributes()
 */
globus_result_t
globus_replica_catalog_collection_delete_attribute(
	globus_replica_catalog_collection_handle_t * handle,
	char * attribute_name,
	char * attribute_value );

/**
 * List the filenames in a collection.
 *
 * Call globus_replica_catalog_attribute_set_free_attribute() after
 * final use of the returned filenames.
 *
 * @ingroup globus_replica_catalog_collection
 *
 * @param handle
 * 	  handle to the collection
 * @param filenames
 * 	  null-terminated array of collection filenames @b returned, may
 * 	  be GLOBUS_NULL if none exist
 *
 * @see globus_replica_catalog_collection_add_filenames(),
 * 	globus_replica_catalog_collection_delete_filenames(),
 * 	globus_replica_catalog_attribute_set_free_attribute() 
 */
globus_result_t
globus_replica_catalog_collection_list_filenames(
	globus_replica_catalog_collection_handle_t * handle,
	char *** filenames );

/**
 * List attribute information (other than filenames) of a collection.
 *
 * Only return attribute information for the given attribute names.
 *
 * Call globus_replica_catalog_attribute_set_destroy() after final use
 * of the returned attributes.
 *
 * @ingroup globus_replica_catalog_collection
 *
 * @param handle
 * 	  handle to the collection
 * @param attribute_names
 * 	  null-terminated array of names of the attributes to get, if
 * 	  GLOBUS_NULL then all attributes are returned
 * @param attributes
 * 	  collection attributes @b returned
 *
 * @see globus_replica_catalog_collection_add_attribute(),
 * 	globus_replica_catalog_collection_delete_attribute(),
 * 	globus_replica_catalog_attribute_set_destroy() 
 */
globus_result_t
globus_replica_catalog_collection_list_attributes(
	globus_replica_catalog_collection_handle_t * handle,
	char ** attribute_names,
	struct globus_replica_catalog_attribute_set_s * attributes );

/**
 * @defgroup globus_replica_catalog_location Location Information Management
 *
 * Manage location information in a collection.
 *
 * These functions manage the information of a given location in a
 * collection represented by a given collection handle.  Except for
 * globus_replica_catalog_location_create(), these functions assume the
 * location already exists.  These functions use a collection handle
 * that has been initialized by either
 * globus_replica_catalog_collection_open() or
 * globus_replica_catalog_collection_create().
 *
 * At a high level, a @e location associates a subset of the filenames
 * in a collection to physical storage.  The filenames dealt with here
 * must exist in the collection, see the @link
 * globus_replica_catalog_collection collection information management
 * @endlink functions.  To associate extra information such as size with
 * a filename see the @link globus_replica_catalog_logicalfile logical
 * file information management @endlink functions.
 *
 * \todo Include a usage example.
 */

/**
 * Create a location.
 *
 * Create a location in a collection and associate it with both a given
 * URL accessor of a corresponding physical collection and a given set
 * of filenames.  All the filenames must already exist in the
 * collection, and the location itself must not already exist.
 *
 * It's assumed a corresponding physical replica has already been
 * created and is accessible through the concatenation of the given URL
 * accessor with each given filename (which may include a path).  This
 * implementation does not check that this is the case.
 *
 * @ingroup globus_replica_catalog_location
 *
 * @param handle
 * 	  handle to the collection
 * @param location_name
 * 	  name of the location to create
 * @param url
 *        URL accessor of the corresponding physical replica,
 *        of the form @c protocol://host:port/path
 * @param filenames
 * 	  null-terminated array of filenames (which may include paths)
 * 	  to add, may be GLOBUS_NULL
 * @param objectclass
 * 	  objectclass name of the location to be created, must be a
 * 	  subclass of #GLOBUS_REPLICA_CATALOG_LOCATION_OBJECTCLASS, if
 * 	  GLOBUS_NULL is given then the default
 * 	  #GLOBUS_REPLICA_CATALOG_LOCATION_OBJECTCLASS is used
 *
 * @see globus_replica_catalog_location_delete()
 */
globus_result_t
globus_replica_catalog_location_create(
	globus_replica_catalog_collection_handle_t * handle,
	char * location_name,
	char * url,
	char ** filenames,
	char * objectclass );

/**
 * Add filenames to a location.
 *
 * If some of the given filenames already exist in the location, and @e
 * add is GLOBUS_TRUE, then add those that don't.  Otherwise, fail if
 * some of the given filenames exist and @e add is GLOBUS_FALSE.  All
 * the filenames must already exist in the collection.
 *
 * It's assumed that the corresponding physical files have already been
 * created and are accessible through the concatenation of the
 * location's URL accessor with each given filename (which may include a
 * path).  This implementation does not check that this is the case.
 *
 * @ingroup globus_replica_catalog_location
 *
 * @param handle
 * 	  handle to the collection
 * @param location_name
 * 	  name of the existing location to add filenames to
 * @param filenames
 * 	  null-terminated array of filenames (which may include paths)
 * 	  to add
 * @param add
 * 	  whether to add given filenames that don't exist in the
 * 	  location even if there are some given filenames that do exist
 *
 * @see globus_replica_catalog_location_delete_filenames(),
 * 	globus_replica_catalog_location_list_filenames()
 */
globus_result_t
globus_replica_catalog_location_add_filenames(
	globus_replica_catalog_collection_handle_t * handle,
	char * location_name,
	char ** filenames,
	globus_bool_t add );

/**
 * Add attribute information (other than filenames) to a location.
 * Does not replace existing attribute values.
 *
 * Note that the arbitrary information associated with a location may
 * apply either to the location itself or to all the filenames in the
 * location depending on the meaning the caller wants to associate with
 * a particular attribute.
 *
 * @ingroup globus_replica_catalog_location
 *
 * @param handle
 * 	  handle to the collection
 * @param location_name
 * 	  name of the existing location to add attribute information to
 * @param attribute_name
 * 	  name of the attribute to add
 * @param attribute_value
 * 	  value of the attribute to add
 *
 * @see globus_replica_catalog_location_delete_attribute(),
 * 	globus_replica_catalog_location_list_attributes()
 */
globus_result_t
globus_replica_catalog_location_add_attribute(
	globus_replica_catalog_collection_handle_t * handle,
	char * location_name,
	char * attribute_name,
	char * attribute_value );

/**
 * Delete a location.
 *
 * @ingroup globus_replica_catalog_location
 *
 * @param handle
 * 	  handle to the collection
 * @param location_name
 * 	  name of the location to delete
 *
 * @see globus_replica_catalog_location_create()
 */
globus_result_t
globus_replica_catalog_location_delete(
	globus_replica_catalog_collection_handle_t * handle,
	char * location_name );

/**
 * Delete filenames from a location.  If one or more of the given
 * filenames is not associated with the location then no deletions are
 * done.
 *
 * @ingroup globus_replica_catalog_location
 *
 * @param handle
 * 	  handle to the collection
 * @param location_name
 * 	  name of the location to delete filenames from
 * @param filenames
 * 	  null-terminated array of filenames to delete
 *
 * @see globus_replica_catalog_location_add_filenames(),
 * 	globus_replica_catalog_location_list_filenames()
 */
globus_result_t
globus_replica_catalog_location_delete_filenames(
	globus_replica_catalog_collection_handle_t * handle,
	char * location_name,
	char ** filenames );

/**
 * Delete attribute information (other than filenames) from a location.
 *
 * @ingroup globus_replica_catalog_location
 *
 * @param handle
 * 	  handle to the collection
 * @param location_name
 * 	  name of the location to delete attribute information from
 * @param attribute_name
 * 	  name of the attribute to delete
 * @param attribute_value
 * 	  value of the attribute to delete, if GLOBUS_NULL then delete
 * 	  all the values associated with @e location_name
 *
 * @see globus_replica_catalog_location_add_attribute(),
 * 	globus_replica_catalog_location_list_attributes()
 */
globus_result_t
globus_replica_catalog_location_delete_attribute(
	globus_replica_catalog_collection_handle_t * handle,
	char * location_name,
	char * attribute_name,
	char * attribute_value );

/**
 * Search for locations with all (or any) of the given filenames.
 *
 * Only return attribute information for the given attribute names.
 *
 * Call globus_replica_catalog_entry_set_destroy() after final use of the
 * returned locations.
 *
 * @ingroup globus_replica_catalog_collection
 *
 * @param handle
 * 	  handle to the collection
 * @param filenames
 * 	  null-terminated array of filenames (which may include paths)
 * 	  to search the locations for
 * @param all
 * 	  whether to return locations that contain @b all the given
 * 	  filenames (GLOBUS_TRUE) or otherwise return locations that
 * 	  contain @b any of the given filenames (GLOBUS_FALSE)
 * @param attribute_names
 * 	  null-terminated array of names of the attributes to get, if
 * 	  GLOBUS_NULL then all attributes are returned
 * @param locations
 * 	  locations @b returned
 *
 * @see globus_replica_catalog_entry_set_destroy()
 */
globus_result_t
globus_replica_catalog_collection_find_locations(
	globus_replica_catalog_collection_handle_t * handle,
	char ** filenames,
	globus_bool_t all,
	char ** attribute_names,
	struct globus_replica_catalog_entry_set_s * locations );

/**
 * List the locations in a collection.
 *
 * For each location only return attribute information for the given
 * attribute names.
 *
 * Call globus_replica_catalog_entry_set_destroy() after final use of the
 * returned locations.
 *
 * @ingroup globus_replica_catalog_collection
 *
 * @param handle
 * 	  handle to the collection
 * @param attribute_names
 * 	  null-terminated array of names of the attributes to get, if
 * 	  GLOBUS_NULL then all attributes are returned
 * @param locations
 * 	  locations @b returned
 *
 * @see globus_replica_catalog_entry_set_destroy()
 */
globus_result_t
globus_replica_catalog_collection_list_locations(
	globus_replica_catalog_collection_handle_t * handle,
	char ** attribute_names,
	struct globus_replica_catalog_entry_set_s * locations );

/**
 * List the filenames in a location.
 *
 * Call globus_replica_catalog_attribute_set_free_attribute() after
 * final use of the returned filenames.
 *
 * @ingroup globus_replica_catalog_location
 *
 * @param handle
 * 	  handle to the collection
 * @param location_name
 * 	  name of the location
 * @param filenames
 *        null-terminated array of location filenames @b returned, may
 *        be GLOBUS_NULL if none exist
 *
 * @see globus_replica_catalog_location_add_filenames(),
 * 	globus_replica_catalog_location_delete_filenames(),
 * 	globus_replica_catalog_attribute_set_free_attribute() 
 */
globus_result_t
globus_replica_catalog_location_list_filenames(
	globus_replica_catalog_collection_handle_t * handle,
	char * location_name,
	char *** filenames );

/**
 * List attribute information (other than filenames) of a location.
 *
 * Only return attribute information for the given attribute names.
 *
 * Call globus_replica_catalog_attribute_set_destroy() after final use
 * of the returned attributes.
 *
 * @ingroup globus_replica_catalog_location
 *
 * @param handle
 * 	  handle to the collection
 * @param location_name
 * 	  name of the location to get the attributes from
 * @param attribute_names
 * 	  null-terminated array of names of the attributes to get, if
 * 	  GLOBUS_NULL then all attributes are returned
 * @param attributes
 * 	  location attributes @b returned
 *
 * @see globus_replica_catalog_location_add_attribute(),
 * 	globus_replica_catalog_location_list_attributes(),
 * 	globus_replica_catalog_attribute_set_destroy() 
 */
globus_result_t
globus_replica_catalog_location_list_attributes(
	globus_replica_catalog_collection_handle_t * handle,
	char * location_name,
	char ** attribute_names,
	struct globus_replica_catalog_attribute_set_s * attributes );

/**
 * @defgroup globus_replica_catalog_logicalfile Logical File Information Management
 *
 * Manage logical file information in a collection.
 *
 * These functions manage the information of a given logicalfile name in
 * a collection represented by a given collection handle.  Except for
 * globus_replica_catalog_logicalfile_create(), these functions assume
 * the logical file already exists.  These functions use a collection
 * handle that has been initialized by either
 * globus_replica_catalog_collection_open() or
 * globus_replica_catalog_collection_create().
 *
 * At a high level, a @e logical @e file associates extra information
 * such as size with a filename in a collection.  The filename must exist
 * in the collection, see the @link globus_replica_catalog_collection
 * collection information management @endlink functions.  To associate a
 * filename in a collection with physical storage see the @link
 * globus_replica_catalog_location location information management
 * @endlink functions.
 *
 * \todo Include a usage example.
 */

/**
 * Create a logical file in a collection.
 *
 * The corresponding collection filename must exist.
 * The logical file must not already exist.
 *
 * @ingroup globus_replica_catalog_logicalfile
 *
 * @param handle
 * 	  handle to the collection
 * @param logicalfile_name
 * 	  name of the logical file to add
 * @param size
 * 	  size in kilobytes of the logical file to add
 * @param objectclass
 * 	  objectclass name of the logical file to be created, must be a
 * 	  subclass of #GLOBUS_REPLICA_CATALOG_LOGICALFILE_OBJECTCLASS, if
 * 	  GLOBUS_NULL is given then the default
 * 	  #GLOBUS_REPLICA_CATALOG_LOGICALFILE_OBJECTCLASS is used
 * 
 * @see globus_replica_catalog_logicalfile_delete()
 */
globus_result_t
globus_replica_catalog_logicalfile_create(
	globus_replica_catalog_collection_handle_t * handle,
	char * logicalfile_name,
	int size,
	char * objectclass );

/**
 * Add attribute information to a logical file.
 * Does not replace existing attribute values.
 *
 * @ingroup globus_replica_catalog_logicalfile
 *
 * @param handle
 * 	  handle to the collection
 * @param logicalfile_name
 * 	  name of the logical file to add attribute information to
 * @param attribute_name
 * 	  name of the attribute to add
 * @param attribute_value
 * 	  value of the attribute to add
 *
 * @see globus_replica_catalog_logicalfile_delete_attribute(),
 * 	globus_replica_catalog_logicalfile_list_attributes()
 */
globus_result_t
globus_replica_catalog_logicalfile_add_attribute(
	globus_replica_catalog_collection_handle_t * handle,
	char * logicalfile_name,
	char * attribute_name,
	char * attribute_value );

/**
 * Delete a logical file.
 *
 * @ingroup globus_replica_catalog_logicalfile
 *
 * @param handle
 * 	  handle to the collection
 * @param logicalfile_name
 * 	  name of the logical file to delete
 * 
 * @see globus_replica_catalog_logicalfile_create()
 */
globus_result_t
globus_replica_catalog_logicalfile_delete(
	globus_replica_catalog_collection_handle_t * handle,
	char * logicalfile_name );

/**
 * Delete attribute information from a logical file.
 *
 * @ingroup globus_replica_catalog_logicalfile
 *
 * @param handle
 * 	  handle to the collection
 * @param logicalfile_name
 * 	  name of the logical file to delete attribute information from
 * @param attribute_name
 * 	  name of the attribute to delete
 * @param attribute_value
 * 	  value of the attribute to delete, if GLOBUS_NULL then delete
 * 	  all the values associated with @e logicalfile_name
 *
 * @see globus_replica_catalog_logicalfile_add_attribute(),
 * 	globus_replica_catalog_logicalfile_list_attributes()
 */
globus_result_t
globus_replica_catalog_logicalfile_delete_attribute(
	globus_replica_catalog_collection_handle_t * handle,
	char * logicalfile_name,
	char * attribute_name,
	char * attribute_value );

/**
 * List the logical files in a collection.
 *
 * For each logical file only return attribute information for the given
 * attribute names.
 *
 * Call globus_replica_catalog_entry_set_destroy() after final use of
 * the returned logical files.
 *
 * @ingroup globus_replica_catalog_collection
 *
 * @param handle
 * 	  handle to the collection
 * @param attribute_names
 * 	  null-terminated array of names of the attributes to get, if
 * 	  GLOBUS_NULL then all attributes are returned
 * @param logicalfiles
 * 	  logical files @b returned
 *
 * @see globus_replica_catalog_entry_set_destroy()
 */
globus_result_t
globus_replica_catalog_collection_list_logicalfiles(
	globus_replica_catalog_collection_handle_t * handle,
	char ** attribute_names,
	struct globus_replica_catalog_entry_set_s * logicalfiles );

/**
 * List the attribute information of a logical file.
 *
 * Only return attribute information for the given attribute names.
 *
 * Call globus_replica_catalog_attribute_set_destroy() after final use
 * of the returned attributes.
 *
 * @ingroup globus_replica_catalog_logicalfile
 *
 * @param handle
 * 	  handle to the collection
 * @param logicalfile_name
 * 	  name of the logical file to get the attributes from
 * @param attribute_names
 * 	  null-terminated array of names of the attributes to get, if
 * 	  GLOBUS_NULL then all attributes are returned
 * @param attributes
 * 	  logical file attributes @b returned
 *
 * @see globus_replica_catalog_logicalfile_add_attribute(),
 * 	globus_replica_catalog_logicalfile_delete_attribute(),
 * 	globus_replica_catalog_attribute_set_destroy() 
 */
globus_result_t
globus_replica_catalog_logicalfile_list_attributes(
	globus_replica_catalog_collection_handle_t * handle,
	char * logicalfile_name,
	char ** attribute_names,
	struct globus_replica_catalog_attribute_set_s * attributes );

#ifdef EXTERN_C_END
EXTERN_C_END
#endif

#endif /* _GLOBUS_REPLICA_CATALOG_H */
