/* $Id: globus_replica_catalog_sets.h,v 1.35 2001/04/27 08:44:27 quesnel Exp $ */

#ifndef GLOBUS_DONT_DOCUMENT_INTERNAL
/**
 * @file
 *
 * Globus Replica Catalog Sets C API Header File
 *
 * This file declares functions, types, and macros that manage sets of
 * entries and attributes.  There are two categories of declarations:
 * @link globus_replica_catalog_attribute_set attribute set management
 * @endlink, and @link globus_replica_catalog_entry_set entry set
 * management @endlink.  The latter relates to the former in that
 * entries have attribute sets associated with them.
 */
#endif /* GLOBUS_DONT_DOCUMENT_INTERNAL */

#ifndef _GLOBUS_REPLICA_CATALOG_SETS_H
#define _GLOBUS_REPLICA_CATALOG_SETS_H

#include "globus_common.h"
#include "globus_replica_catalog.h"
#include "lber.h"
#include "ldap.h"

#ifdef __cplusplus
extern "C" {
#endif

/* struct from globus_replica_catalog.h */
struct globus_replica_catalog_collection_handle_s;

/**
 * @defgroup globus_replica_catalog_attribute_set Attribute Set Management
 *
 * Manage a set of attributes.
 *
 * See globus_replica_catalog_attribute_set_s for a note about the use
 * of an attribute set.
 *
 * As an example let's iterate over an attribute set and print out the
 * name and the values of each attribute in turn.  First, declare what
 * we need and initialize the attribute set:
 *
 * @code
 * globus_replica_catalog_attribute_set_t attributes;
 * char *name, **values;
 * int i;
 * globus_replica_catalog_attribute_set_init( &attributes );
 * @endcode
 *
 * Open a connection to the collection and call a query function that
 * returns an attribute set (omitted), go to the first attribute, loop
 * while there are more attributes, and get the name and values of the
 * current attribute:
 *
 * @code
 * globus_replica_catalog_attribute_set_first( &attributes );
 * while (globus_replica_catalog_attribute_set_more( &attributes )) {
 *     globus_replica_catalog_attribute_set_get( &attributes, &name, &values );
 * @endcode
 *
 * Print out the name and values (note that the array of values is
 * null-terminated):
 *
 * @code
 *     printf( "%s:\n", name );
 *     for (i = 0; values[i]; i++ ){
 *         printf( "\t%s\n", values[i] );
 *     }
 * @endcode
 *
 * Now, free the resulting attribute name and values before reuse,
 * advance to the next attribute, then clear the attribute set after
 * final use:
 *
 * @code
 *     globus_replica_catalog_attribute_set_free_attribute( name, values );
 *     globus_replica_catalog_attribute_set_next( &attributes ) ) {
 * }
 * globus_replica_catalog_attribute_set_destroy( &attributes );
 * @endcode
 *
 * Finally, close the connection to the collection (omitted).
 */

/**
 * Set of attributes.
 *
 * Before use, call globus_replica_catalog_attribute_set_init(), and one of
 * globus_replica_catalog_entry_set_get_attributes(),
 * globus_replica_catalog_collection_list_attributes(),
 * globus_replica_catalog_location_list_attributes(), or
 * globus_replica_catalog_logicalfile_list_attributes()
 * to set the contents of the attribute set.
 *
 * After use, call globus_replica_catalog_attribute_set_destroy().
 *
 * Use constitutes calling globus_replica_catalog_attribute_set_first(),
 * then some combination of globus_replica_catalog_attribute_set_more(),
 * globus_replica_catalog_attribute_set_next(), and/or
 * globus_replica_catalog_attribute_set_get().
 *
 * @ingroup globus_replica_catalog_attribute_set
 */
typedef struct globus_replica_catalog_attribute_set_s
{
#ifndef GLOBUS_DONT_DOCUMENT_INTERNAL

    /** collection handle */
    struct globus_replica_catalog_collection_handle_s * collection;

    /** current entry */
    LDAPMessage * entry;

    /** chain of entries - this is included because sometimes the
     * attributes are responsible for deallocating the entries */
    LDAPMessage * result;

    /** ber encoding of the entry */
    BerElement * ber;

    /** current attribute name */
    char * attribute_name;

#endif /* GLOBUS_DONT_DOCUMENT_INTERNAL */
}
globus_replica_catalog_attribute_set_t;

/**
 * Initialize an attribute set.
 *
 * Must be followed by a call such as
 * globus_replica_catalog_entry_set_get_attributes() that sets the
 * contents of the attribute set.  Call
 * globus_replica_catalog_attribute_set_destroy() after use of the
 * attribute set.
 *
 * @ingroup globus_replica_catalog_attribute_set
 *
 * @param attributes
 * 	  attribute set @b returned, caller-allocated
 */
globus_result_t
globus_replica_catalog_attribute_set_init(
	globus_replica_catalog_attribute_set_t * attributes );

/**
 * Clear an attribute set.
 *
 * Must be called after use of an attribute set.
 *
 * @ingroup globus_replica_catalog_attribute_set
 *
 * @param attributes
 * 	  attribute set
 *
 * @see globus_replica_catalog_attribute_set_init()
 */
globus_result_t
globus_replica_catalog_attribute_set_destroy(
	globus_replica_catalog_attribute_set_t * attributes );

/**
 * Set the current attribute to the first attribute in an attribute set.
 *
 * Must be preceded by a call such as
 * globus_replica_catalog_entry_set_get_attributes() that sets the
 * contents of the attribute set.  Should be followed by a check to see
 * if there are any attributes, i.e.,
 * globus_replica_catalog_attribute_set_more().
 *
 * @ingroup globus_replica_catalog_attribute_set
 *
 * @param attributes
 * 	  attribute set
 *
 * @see globus_replica_catalog_attribute_set_next(),
 * 	globus_replica_catalog_attribute_set_more()
 */
globus_result_t
globus_replica_catalog_attribute_set_first(
	globus_replica_catalog_attribute_set_t * attributes );

/**
 * Set the current attribute to the next attribute in an attribute set.
 *
 * Must have been preceded at some point by a call to
 * globus_replica_catalog_attribute_set_first().
 * Should be followed by a test of
 * globus_replica_catalog_attribute_set_more().
 *
 * @ingroup globus_replica_catalog_attribute_set
 *
 * @param attributes
 * 	  attribute set
 */
globus_result_t
globus_replica_catalog_attribute_set_next(
	globus_replica_catalog_attribute_set_t * attributes );

/**
 * Whether there are more attributes in an attribute set, inclusive of
 * the current attribute.
 *
 * Must have been preceded at some point by a call to
 * globus_replica_catalog_attribute_set_first().
 *
 * @ingroup globus_replica_catalog_attribute_set
 *
 * @param attributes
 * 	  attribute set
 *
 * @retval GLOBUS_TRUE
 * 	   there are more attributes
 * @retval GLOBUS_FALSE
 * 	   there are no more attributes
 *
 * @see globus_replica_catalog_attribute_set_next()
 */
globus_bool_t
globus_replica_catalog_attribute_set_more(
	globus_replica_catalog_attribute_set_t * attributes );

/**
 * Get the name of the current attribute in an attribute set.
 *
 * Must have been preceded at some point by a call to
 * globus_replica_catalog_attribute_set_first().  Call
 * globus_libc_free() after final use of the returned attribute name.
 *
 * @ingroup globus_replica_catalog_attribute_set
 *
 * @param attributes
 * 	  attribute set
 * @param attribute_name
 * 	  name of the current attribute @b returned
 *
 * @see globus_replica_catalog_attribute_set_next()
 */
globus_result_t
globus_replica_catalog_attribute_set_get_name(
	globus_replica_catalog_attribute_set_t * attributes,
	char ** attribute_name );

/**
 * Get the values of the current attribute in an attribute set.
 *
 * Must have been preceded at some point by a call to
 * globus_replica_catalog_attribute_set_first().  Call
 * globus_replica_catalog_attribute_set_free_values() after final use of
 * the returned name and values.
 *
 * @ingroup globus_replcia_catalog_attribute_set
 *
 * @param attributes
 * 	  attribute set
 * @param attribute_values
 * 	  null-terminated array of the values of the current attribute
 * 	  @b returned
 *
 * @see globus_replica_catalog_attribute_set_next()
 */
globus_result_t
globus_replica_catalog_attribute_set_get_values(
	globus_replica_catalog_attribute_set_t * attributes,
	char *** attribute_values );

/**
 * Free attribute values.
 *
 * Must be called subsequent to final use of values that have been
 * returned by globus_replica_catalog_attribute_set_get_values(),
 * globus_replica_catalog_collection_list_filenames(),
 * globus_replica_catalog_location_list_filenames(), or
 * globus_replica_catalog_collection_list_filenames().
 *
 * @param attribute_values
 * 	  array of previously returned attribute values
 */
globus_result_t
globus_replica_catalog_attribute_set_free_values(
	char ** attribute_values );

/**
 * @defgroup globus_replica_catalog_entry_set Entry Set Management
 *
 * Manage a set of entries.
 *
 * See globus_replica_catalog_entry_set_s for a note about the use of an
 * entry set.
 *
 * As an example let's iterate over an entry set.  First, declare and
 * initialize an entry set and attribute set for use:
 *
 * @code
 * globus_replica_catalog_entry_set_t entries;
 * globus_replica_catalog_attribute_set_t attributes;
 * globus_replica_catalog_entry_set_init( &entries );
 * globus_replica_catalog_attribute_set_init( &attributes );
 * @endcode
 *
 * Open a connection to a collection and call a query function that
 * returns an entry set (omitted).  Go to the first entry, loop while
 * there are more entries, and get the attributes associated with the
 * current entry: 
 *
 * @code
 * globus_replica_catalog_entry_set_first( &entries );
 * while (globus_replica_catalog_entry_set_more( &entries )) {
 *     globus_replica_catalog_entry_set_get_attributes( &entries, &attributes );
 * @endcode
 *
 * Now we can do something with the resulting attribute set (omitted).
 * Remember to clear the attribute set before reuse, advance to the next
 * entry, and clear the entry set after final use:
 *
 * @code
 *     globus_replica_catalog_attribute_set_destroy( &attributes );
 *     globus_replica_catalog_entry_set_next( &entries );
 * }
 * globus_replica_catalog_entry_set_destroy( &entries );
 * @endcode
 *
 * Finally, close the connection to the collection (omitted).
 */

/**
 * Set of entries.
 *
 * Before use, call globus_replica_catalog_entry_set_init(), and one of
 * globus_replica_catalog_collection_find_locations(),
 * globus_replica_catalog_collection_list_locations(), or
 * globus_replica_catalog_collection_list_logicalfiles()
 * to set the contents of the entry set.
 *
 * After use, call globus_replica_catalog_entry_set_destroy().
 *
 * Use constitutes calling globus_replica_catalog_entry_set_first(),
 * then some combination of globus_replica_catalog_entry_set_more(),
 * globus_replica_catalog_entry_set_next(),
 * globus_replica_catalog_entry_set_count(), and/or
 * globus_replica_catalog_entry_set_get_attributes().
 *
 * @ingroup globus_replica_catalog_entry_set
 */
typedef struct globus_replica_catalog_entry_set_s
{
#ifndef GLOBUS_DONT_DOCUMENT_INTERNAL

    /** collection handle */
    struct globus_replica_catalog_collection_handle_s * collection;

    /** whether the entry functions are responsible for closing the
     * collection */
    globus_bool_t collection_close;

    /** chain of entries */
    LDAPMessage * result;

    /** current entry */
    LDAPMessage * entry;

#endif /* GLOBUS_DONT_DOCUMENT_INTERNAL */
}
globus_replica_catalog_entry_set_t;

/**
 * Initialize an entry set.
 *
 * Must be followed by a call such as
 * globus_replica_catalog_collection_list_locations() that sets the
 * contents of the entry set.  Call
 * globus_replica_catalog_entry_set_destroy() after use of the entry set.
 *
 * @ingroup globus_replica_catalog_entry_set
 *
 * @param entries
 * 	  entry set @b returned, caller-allocated
 */
globus_result_t
globus_replica_catalog_entry_set_init(
	globus_replica_catalog_entry_set_t * entries );

/**
 * Clear an entry set.
 *
 * Must be called after use of an entry set.
 *
 * @ingroup globus_replica_catalog_entry_set
 *
 * @param entries
 * 	  entry set
 *
 * @see globus_replica_catalog_entry_set_init()
 */
globus_result_t
globus_replica_catalog_entry_set_destroy(
	globus_replica_catalog_entry_set_t * entries );

/**
 * Set the current entry to the first entry of an entry set.
 *
 * Must be preceded by a call such as
 * globus_replica_catalog_collection_list_locations() that sets the
 * contents of the entry set.  Should be followed by a check to see if
 * there are any entries, i.e., globus_replica_catalog_entry_set_more().
 *
 * @ingroup globus_replica_catalog_entry_set
 *
 * @param entries
 * 	  entry set
 *
 * @see globus_replica_catalog_entry_set_next()
 */
globus_result_t
globus_replica_catalog_entry_set_first(
	globus_replica_catalog_entry_set_t * entries );

/**
 * Set the current entry to the next entry of the entry set.
 *
 * Must have been preceded at some point by a call to
 * globus_replica_catalog_entry_set_first().
 * Should be followed by a test of
 * globus_replica_catalog_entry_set_more().
 *
 * @ingroup globus_replica_catalog_entry_set
 *
 * @param entries
 * 	  entry set
 */
globus_result_t
globus_replica_catalog_entry_set_next(
	globus_replica_catalog_entry_set_t * entries );

/**
 * Whether there are more entries in an entry set, inclusive of the
 * current entry.
 *
 * Must have been preceded at some point by a call to
 * globus_replica_catalog_entry_set_first().
 *
 * @ingroup globus_replica_catalog_entry_set
 *
 * @param entries
 * 	  entry set
 *
 * @retval GLOBUS_TRUE
 * 	   there are more entries
 * @retval GLOBUS_FALSE
 * 	   there are @b no more entries
 *
 * @see globus_replica_catalog_entry_set_next()
 */
globus_bool_t
globus_replica_catalog_entry_set_more(
	globus_replica_catalog_entry_set_t * entries );

/**
 * Get the number of entries in the entry set.
 *
 * Must be preceded by a call such as
 * globus_replica_catalog_collection_list_locations() that sets the
 * contents of the entry set.
 *
 * @ingroup globus_replica_catalog_entry_set
 *
 * @param entries
 * 	  entry set
 * @param count
 * 	  number of elements @b returned, caller-allocated
 */
globus_result_t
globus_replica_catalog_entry_set_count(
	globus_replica_catalog_entry_set_t * entries,
	int * count );

/**
 * Get the DN of the current entry in an entry set.
 *
 * Must have been preceded at some point by a call to
 * globus_replica_catalog_entry_set_first().
 * Call globus_replica_catalog_attribute_set_free_dn() after use of the
 * returned entry DN.
 *
 * @ingroup globus_replica_catalog_entry_set
 *
 * @param entries
 * 	  entry set
 * @param entry_dn
 * 	  entry DN @b returned
 *
 * @see globus_replica_catalog_entry_set_next()
 */
globus_result_t
globus_replica_catalog_entry_set_get_dn(
	globus_replica_catalog_entry_set_t * entries,
	char ** entry_dn );

/**
 * Get the URL of the current entry in an entry set.
 *
 * Must have been preceded at some point by a call to
 * globus_replica_catalog_entry_set_first().
 * Call globus_libc_free() after use of the
 * returned entry DN.
 *
 * @ingroup globus_replica_catalog_entry_set
 *
 * @param entries
 * 	  entry set
 * @param entry_url
 * 	  entry URL @b returned
 *
 * @see globus_replica_catalog_entry_set_next()
 */
globus_result_t
globus_replica_catalog_entry_set_get_url(
	globus_replica_catalog_entry_set_t * entries,
	char ** entry_url );

/**
 * Free attribute DN.
 *
 * Must be called subsequent to final use of the DN
 * returned by globus_replica_catalog_entry_set_get_dn().
 *
 * @ingroup globus_replica_catalog_entry_set
 *
 * @param entry_dn
 * 	  entry DN
 */
globus_result_t
globus_replica_catalog_entry_set_free_dn(
	char * entry_dn );

/**
 * Get the name of the current entry in an entry set.
 *
 * Must have been preceded at some point by a call to
 * globus_replica_catalog_entry_set_first().  Call globus_libc_free()
 * after use of the returned entry name.
 *
 * @ingroup globus_replica_catalog_entry_set
 *
 * @param entries
 * 	  entry set
 * @param entry_name
 * 	  entry name @b returned
 *
 * @see globus_replica_catalog_entry_set_next()
 */
globus_result_t
globus_replica_catalog_entry_set_get_name(
	globus_replica_catalog_entry_set_t * entries,
	char ** entry_name );

/**
 * Get the attribute set of the current entry in an entry set.
 *
 * Must have been preceded at some point by a call to
 * globus_replica_catalog_entry_set_first().  Call
 * globus_replica_catalog_attribute_set_destroy() after use of the
 * returned attribute set.
 *
 * @ingroup globus_replica_catalog_entry_set
 *
 * @param entries
 * 	  entry set
 * @param attributes
 * 	  attribute set @b returned
 *
 * @see globus_replica_catalog_entry_set_next()
 */
globus_result_t
globus_replica_catalog_entry_set_get_attributes(
	globus_replica_catalog_entry_set_t * entries,
	globus_replica_catalog_attribute_set_t * attributes );

#ifdef __cplusplus
}
#endif

#endif /* _GLOBUS_REPLICA_CATALOG_SETS_H */
