/* $Id: globus_replica_management.h,v 1.35 2001/07/30 22:15:17 quesnel Exp $ */

#ifndef GLOBUS_DONT_DOCUMENT_INTERNAL
/**
 * @file
 *
 * Globus Replica Management C API Header File
 *
 * This file declares types and functions that manage the placement of
 * files on remote storage systems, and that manage the information
 * about these files and their locations in a replica catalog.
 */
#endif /* GLOBUS_DONT_DOCUMENT_INTERNAL */

/**
 * @mainpage Globus Replica Management C API
 * @anchor globus_replica_management_api
 *
 * The Globus Replica Management C API provides operations that manage
 * the placement of files on remote storage systems and manages the
 * information about these files and their locations in a replica
 * catalog.  There are functions and structures for @link
 * globus_replica_management_session session management @endlink, @link
 * globus_replica_management_creation catalog creation @endlink, and
 * @link globus_replica_management_file file maintenance @endlink.
 *
 * Here a @e replica is simply a @e managed @e copy of a file.  Within
 * the replica management system we can:
 * 	- register/publish physical files to be managed;
 * 	- create replicas from other existing replicas; and
 * 	- delete replicas.
 *
 * Note that:
 * 	- there is no consistency implied between replicas
 * 	- once a logical file name is associated with a physical file,
 * 	  no modifications are allowed to that file; and
 * 	- replica management functions do not guarantee that replicas
 * 	  are not modified by methods outside the context of the replica
 * 	  management system.
 *
 * This library uses the 
 * @ref globus_replica_catalog_api "Globus Replica Catalog C API"
 * to manipulate information in a replica
 * catalog about the location of managed files.  We also rely on the 
 * @ref globus_ftp_client_api "Globus FTP Client API"
 * for file transfer via the GridFTP protocol.
 *
 * Group-oriented access control is being specified and developed
 * separately from this API.  A community access service will be
 * used in future implementations of this library.  For now, GSI or
 * plaintext authentication will be used when connecting to a storage
 * server or replica catalog.
 */

#ifndef GLOBUS_DONT_DOCUMENT_INTERNAL

/* defines for testing the rollback/restart functionality */

#ifndef GLOBUS_REPLICA_CATALOG_LOCATION_DELETE_FILENAMES
#define GLOBUS_REPLICA_CATALOG_LOCATION_DELETE_FILENAMES globus_replica_catalog_location_delete_filenames
#endif

#ifndef GLOBUS_I_REPLICA_CATALOG_ROLLBACK_INFO_UPDATE
#define GLOBUS_I_REPLICA_CATALOG_ROLLBACK_INFO_UPDATE globus_i_replica_catalog_rollback_info_update
#endif

#ifndef GLOBUS_I_REPLICA_CATALOG_ROLLBACK_INFO_DELETE
#define GLOBUS_I_REPLICA_CATALOG_ROLLBACK_INFO_DELETE globus_i_replica_catalog_rollback_info_delete
#endif

#ifndef GLOBUS_REPLICA_CATALOG_COLLECTION_ADD_FILENAMES
#define GLOBUS_REPLICA_CATALOG_COLLECTION_ADD_FILENAMES globus_replica_catalog_collection_add_filenames
#endif

#ifndef GLOBUS_REPLICA_CATALOG_LOCATION_ADD_FILENAMES
#define GLOBUS_REPLICA_CATALOG_LOCATION_ADD_FILENAMES globus_replica_catalog_location_add_filenames
#endif

#define GLOBUS_L_REPLICA_MANAGEMENT_BUFFER_SIZE 1024

#endif /* GLOBUS_DONT_DOCUMENT_INTERNAL */

#ifndef _GLOBUS_REPLICA_MANAGEMENT_H
#define _GLOBUS_REPLICA_MANAGEMENT_H

#include "globus_error_rollback_contact.h"
#include "globus_replica_catalog.h"
#include "globus_ftp_client.h"
#include "globus_ftp_client_plugin.h"
#include "globus_gass_copy.h"

#ifdef EXTERN_C_BEGIN
EXTERN_C_BEGIN
#endif

#ifndef GLOBUS_DONT_DOCUMENT_INTERNAL

/** Replica management error format. */
#define GLOBUS_REPLICA_MANAGEMENT_ERROR(text, function) \
        globus_error_put( \
	    globus_error_construct_string( \
		GLOBUS_REPLICA_MANAGEMENT_MODULE, \
		GLOBUS_NULL, \
		"[%s]  %s at %s\n", \
		GLOBUS_REPLICA_MANAGEMENT_MODULE->module_name, \
                text, \
		function)) 

/** Replica management rollback contact error format. */
#define GLOBUS_REPLICA_MANAGEMENT_ROLLBACK_ERROR(contact_string, base_error) \
        globus_error_put( \
	    globus_error_construct_rollback_contact( \
		GLOBUS_REPLICA_MANAGEMENT_MODULE, \
		base_error, \
		"%s", \
		contact_string)) 

#endif /* GLOBUS_DONT_DOCUMENT_INTERNAL */

/**
 * @defgroup globus_replica_management_session Session Management
 *
 * Manage replica management session configuration and state.
 *
 * These functions create, configure, and destroy session handles and
 * their associated attributes.
 */

/**
 * Module descriptor.
 *
 * The Globus replica management library uses the standard module
 * activation and deactivation API to initialize its state. Before
 * any replica management functions are called, the module must be
 * activated
 *
 * @code
 *    globus_module_activate(GLOBUS_REPLICA_MANAGEMENT_MODULE);
 * @endcode
 *
 * This function returns GLOBUS_SUCCESS if the management library was
 * successfully initialized. This may be called multiple times.
 *
 * To deactivate the management library, the following must be called
 *
 * @code
 *    globus_module_deactivate(GLOBUS_REPLICA_MANAGEMENT_MODULE);
 * @endcode
 *
 * @ingroup globus_replica_management_session
 */
#define GLOBUS_REPLICA_MANAGEMENT_MODULE (&globus_i_replica_management_module)
extern globus_module_descriptor_t globus_i_replica_management_module;

/**
 * Session attributes.
 *
 * Session attributes are used to configure a session handle with, say,
 * security information or performance tuning hints.  The set of handle
 * attribute options will grow over time and may include attributes
 * attribute options will grow over time and may include attributes
 * inherited from @c globus_ftp_client and @c globus_replica_catalog .
 *
 * @ingroup globus_replica_management_session
 *
 * @see globus_replica_management_handle_t
 * @see globus_replica_management_handleattr_init(),
 *	globus_replica_management_handleattr_destroy(),
 *      globus_replica_management_handleattr_copy(),
 *	globus_replica_management_handleattr_set_caching(),
 *      globus_replica_management_handleattr_set_ftp_update_frequency(),
 *      globus_replica_management_handleattr_set_ftp_handleattr(),
 *      globus_replica_management_handleattr_set_third_party_transfer()
 */
typedef struct globus_replica_management_handleattr_s
{
#ifndef GLOBUS_DONT_DOCUMENT_INTERNAL

    globus_bool_t                       caching;

    globus_ftp_client_handleattr_t      ftp_handle_attrs;

    /** frequency of rollback and restart info update with a 5 second
     * resolution */
    unsigned int                        ftp_update_frequency;

    /** whether to initiate an ftp third party transfer or to buffer the
     * transfer locally */
    globus_bool_t			third_party_transfer;

#endif /* GLOBUS_DONT_DOCUMENT_INTERNAL */
}
globus_replica_management_handleattr_t;

/**
 * Initialize the session attributes.
 *
 * @ingroup globus_replica_management_session
 *
 * @param attrs
 * 	  the handle attributes to initialize
 *
 * @see globus_replica_management_handleattr_destroy()
 */
globus_result_t
globus_replica_management_handleattr_init(
    globus_replica_management_handleattr_t *    attrs );

/**
 * Destroy the session attributes.
 *
 * @ingroup globus_replica_management_session
 *
 * @param attrs
 * 	  the handle attributes to destroy
 *
 * @see globus_replica_management_handleattr_init()
 */
globus_result_t
globus_replica_management_handleattr_destroy(
    globus_replica_management_handleattr_t *    attrs );

/**
 * Copy the session attributes.
 *
 * @ingroup globus_replica_management_session
 *
 * @param dest
 * 	  the handle attributes to copy into, should not have been
 * 	  previously initialized
 * @param src
 *        the handle attributes to copy from
 *
 * @see globus_replica_management_handleattr_init()
 */
globus_result_t
globus_replica_management_handleattr_copy(
    globus_replica_management_handleattr_t *    dest,
    globus_replica_management_handleattr_t *    src);

/**
 * Set the catalog connection caching behavior in the session
 * attributes.
 *
 * @ingroup globus_replica_management_session
 *
 * @param attrs
 * 	  the handle attributes
 * @param caching
 *        whether to enable caching
 */
globus_result_t
globus_replica_management_handleattr_set_caching(
    globus_replica_management_handleattr_t *    attrs,
    globus_bool_t                               caching);

/**
 * Get the catalog connection caching behavior from the session
 * attributes.
 *
 * @ingroup globus_replica_management_session
 *
 * @param attrs
 * 	  the handle attributes
 * @param caching
 *        whether to enable caching @b returned
 */
globus_result_t
globus_replica_management_handleattr_get_caching(
    globus_replica_management_handleattr_t *    attrs,
    globus_bool_t *                             caching);

/**
 * Set the rollback and restart information update frequency for GridFTP
 * transfers in the session attributes.
 *
 * @ingroup globus_replica_management_session
 *
 * @param attrs
 * 	  the handle attributes
 * @param frequency
 *        how often the rollback and restart information is generated
 *        during a transfer is pushed out to the rollback and restart
 *        information entry, expressed as a multiple of 5 second
 *        intervals
 */
globus_result_t
globus_replica_management_handleattr_set_ftp_update_frequency(
    globus_replica_management_handleattr_t *    attrs,
    unsigned int                                frequency);

/**
 * Get the catalog ftp update frequency behavior from the session 
 * attributes.
 *
 * @ingroup globus_replica_management_session
 *
 * @param attrs
 * 	  the handle attributes
 * @param frequency
 *        how often the rollback and restart information is generated
 *        during a transfer is pushed out to the rollback and restart
 *        information entry, expressed as a multiple of 5 second
 *        intervals
 */
globus_result_t
globus_replica_management_handleattr_get_ftp_update_frequency(
    globus_replica_management_handleattr_t *    attrs,
    unsigned int *                              frequency);

/**
 * Set the GridFTP handle attributes in the session attributes.
 * These attributes are used when initializing the GridFTP handle.
 *
 * @ingroup globus_replica_management_session
 *
 * @param attrs
 * 	  the handle attributes
 * @param ftp_handle_attrs
 *        the GridFTP handle attributes
 */
globus_result_t
globus_replica_management_handleattr_set_ftp_handleattr(
    globus_replica_management_handleattr_t *    attrs,
    globus_ftp_client_handleattr_t *            ftp_handle_attrs);

/**
 * Get the GridFTP handle attributes from the session attributes.
 * These attributes are used when initializing the GridFTP handle.
 *
 * @ingroup globus_replica_management_session
 *
 * @param attrs
 * 	  the handle attributes
 * @param ftp_handle_attrs
 *        the GridFTP handle attributes to be initialized
 */
globus_result_t
globus_replica_management_handleattr_get_ftp_handleattr(
    globus_replica_management_handleattr_t *    attrs,
    globus_ftp_client_handleattr_t *            ftp_handle_attrs);

/**
 * Set whether to initiate an FTP third party transfer or to buffer the
 * transfer locally.
 * The default is to use third party transfer.
 *
 * @ingroup globus_replica_management_session
 *
 * @param attrs
 * 	  the handle attributes
 * @param third_party_transfer
 *        whether to initiate an FTP third party transfer
 */
globus_result_t
globus_replica_management_handleattr_set_third_party_transfer(
    globus_replica_management_handleattr_t *    attrs,
    globus_bool_t                               third_party_transfer);

/**
 * Get whether to initiate an FTP third party transfer or to buffer the
 * transfer locally.
 *
 * @ingroup globus_replica_management_session
 *
 * @param attrs
 * 	  the handle attributes
 * @param third_party_transfer
 *        whether to initiate an FTP third party transfer, @b returnded
 */
globus_result_t
globus_replica_management_handleattr_get_third_party_transfer(
    globus_replica_management_handleattr_t *    attrs,
    globus_bool_t *                             third_party_transfer);

/**
 * Session handle.
 *
 * The session handle is threaded through all replica management
 * functions.  It contains configuration and state information about
 * replica management operations that are in progress, and allows for
 * session state (for example, GridFTP and LDAP connections) to be
 * cached between operations.
 *
 * @ingroup globus_replica_management_session
 *
 * @see globus_replica_management_handle_init(),
 *	globus_replica_management_handle_destroy()
 */
struct globus_replica_management_handle_s;

typedef struct globus_replica_management_plugin_data_s
{
    struct globus_replica_management_handle_s *     handle;
    char *                                          location_name;
    char *                                          filename;
    globus_ftp_client_restart_marker_t              marker;
    int                                             update_count;
    int                                             final_response_count;
}
globus_replica_management_plugin_data_t;

typedef struct globus_replica_management_handle_s
{
    globus_mutex_t                                      mutex;
    globus_list_t *                                     replica_cache;
    globus_replica_catalog_collection_handle_t *        replica_handle;
    globus_list_t *                                     cache_entry;
    globus_ftp_client_handle_t                          ftp_handle;
    globus_ftp_client_plugin_t  		        plugin;
    globus_replica_management_plugin_data_t             plugin_data;
    globus_replica_management_handleattr_t              attrs;
    globus_list_t *                                     list_elem;
    globus_gass_copy_handle_t                           gass_copy_handle;
}
globus_replica_management_handle_t;


/**
 * Initialize the session handle.
 *
 * @ingroup globus_replica_management_session
 *
 * @param handle
 * 	  the session handle to initialize
 *
 * @see globus_replica_management_handle_destroy()
 */
globus_result_t
globus_replica_management_handle_init(
    globus_replica_management_handle_t *        handle,
    globus_replica_management_handleattr_t *    attrs);

/**
 * Destroy the session handle.  After the handle has been destroyed
 * passing it to any other function than
 * globus_replica_management_handle_init() will result in undefined
 * behavior (i.e., core dump).
 *
 * @ingroup globus_replica_management_session
 *
 * @param handle
 * 	  the session handle to destroy.
 *
 * @see globus_replica_management_handle_init()
 */
globus_result_t
globus_replica_management_handle_destroy(
    globus_replica_management_handle_t *    handle);

/**
 * @defgroup globus_replica_management_creation Catalog Creation
 *
 * Create replica catalog collections and locations, populating them
 * with file information from existing storage systems.
 */

/**
 * Create an empty collection with no filenames and default access
 * control rights as defined by the administrator of the replica
 * catalog.
 *
 * The caller must be authorized to create the collection entry in the
 * replica catalog. 
 *
 * @ingroup globus_replica_management_creation
 *
 * @param handle
 * 	  the session handle
 * @param collection_url
 * 	  the LDAP URL of the collection to add 
 * @param collection_attrs
 *        replica catalog collection attributes
 */
globus_result_t
globus_replica_management_collection_create(
    globus_replica_management_handle_t *              handle,
    char *                                            collection_url,
    globus_replica_catalog_collection_handleattr_t *  collection_attrs);

/**
 * Delete a collection with no filenames
 *
 * The caller must be authorized to delete the collection entry in the
 * replica catalog. 
 *
 * @ingroup globus_replica_management_creation
 *
 * @param handle
 * 	  the session handle
 * @param collection_url
 * 	  the LDAP URL of the collection to delete 
 * @param collection_attrs
 *        replica catalog collection attributes
 */
globus_result_t
globus_replica_management_collection_delete(
    globus_replica_management_handle_t *              handle,
    char *                                            collection_url,
    globus_replica_catalog_collection_handleattr_t *  collection_attrs);

/**
 * Create a storage system location in the replica catalog without any
 * associated filenames.
 *
 * The caller must be authorized to create the storage system directory
 * and to add a location entry to the collection in the replica catalog.
 *
 * @ingroup globus_replica_management_creation
 *
 * @param handle
 * 	  the session handle
 * @param collection_url
 * 	  the LDAP URL, for example ldap://host:port/basedn, of the
 *        collection to add the location to
 * @param collection_attrs
 *        replica catalog collection attributes. This parameter may be
 *        GLOBUS_NULL, in which case default attributes are assumed
 * @param location_name
 * 	  the unique name of a location to add within the replica
 * 	  catalog collection
 * @param location_attrs
 *        GridFTP client attributes associated with the location,
 *        may be GLOBUS_NULL, in which case  default attributes are
 *        assumed
 * @param root_dir_url
 * 	  the URL constructor or URL of the root directory in the
 * 	  directory system where the files of this collection will
 * 	  reside, for example, @c "gsiftp://host:port/dir"
 */
globus_result_t
globus_replica_management_location_create(
    globus_replica_management_handle_t *              handle,
    char *                                            collection_url,
    globus_replica_catalog_collection_handleattr_t *  collection_attrs,
    char *                                            location_name,
    globus_ftp_client_operationattr_t *               location_attrs,
    char *                                            root_dir_url);

/**
 * Delete a storage system location from the replica catalog.
 *
 * The caller must be authorized to remove the location entry from the
 * collection in the replica catalog. Note that this call will fail if
 * there are any files associated with the location and that the
 * associated storage system will not be affected.
 *
 * @ingroup globus_replica_management_creation
 *
 * @param handle
 * 	  the session handle
 * @param collection_url
 * 	  the LDAP URL, for example ldap://host:port/basedn, of the
 *        collection to remove the location from
 * @param collection_attrs
 *        replica catalog collection attributes. This parameter may be
 *        GLOBUS_NULL, in which case default attributes are assumed
 * @param location_name
 * 	  the name of a location to remove from the replica
 * 	  catalog collection
 */
globus_result_t
globus_replica_management_location_delete(
    globus_replica_management_handle_t *              handle,
    char *                                            collection_url,
    globus_replica_catalog_collection_handleattr_t *  collection_attrs,
    char *                                            location_name);

/**
 * Register an existing file in a storage system into an existing
 * replica catalog collection and a location.
 *
 * The file already exists in a storage system referred to by a replica
 * catalog location, no copying is done.
 *
 * The caller must have authorization to add a filename to the replica
 * catalog collection and location, and to read the file from the
 * storage system.
 *
 * @ingroup globus_replica_management_creation
 *
 * @param handle 
 * 	  the session handle
 * @param collection_url
 * 	  the LDAP URL of an existing replica catalog collection to add
 * 	  the file to
 * @param collection_attrs
 *        replica catalog collection attributes. This parameter may be
 *        GLOBUS_NULL, in which case default attributes are assumed
 * @param location_name
 * 	  the name of an existing replica catalog location within the
 * 	  replica catalog collection to add the file to
 * @param location_attrs
 *        GridFTP client attributes associated with the location,
 *        may be GLOBUS_NULL, in which case  default attributes are
 *        assumed
 * @param filename
 * 	  the logical file name of the file to add must exist in the
 * 	  storage system directory referred to by the replica catalog
 * 	  location's URL constructor
 */
globus_result_t
globus_replica_management_file_register(
    globus_replica_management_handle_t *              handle,
    char *                                            collection_url,
    globus_replica_catalog_collection_handleattr_t *  collection_attrs,
    char *                                            location_name,
    globus_ftp_client_operationattr_t *               location_attrs,
    char *                                            filename);

/**
 * Storage system overwrite behavior.
 *
 * @ingroup globus_replica_management_creation
 */
typedef enum globus_replica_management_overwrite_behavior_e
{
    /**
     * If the destination file exists then disallow overwrite.
     */
    GLOBUS_REPLICA_MANAGEMENT_OVERWRITE_DISALLOW,

    /**
     * If the destination file exists then keep the copy on the remote
     * system until the source file is fully copied.
     */
    GLOBUS_REPLICA_MANAGEMENT_OVERWRITE_SAFE,

    /**
     * If the destination file exists then copy to the source file by
     * overwriting.
     */
    GLOBUS_REPLICA_MANAGEMENT_OVERWRITE_UNSAFE
}
globus_replica_management_overwrite_behavior_t;

/**
 * Preregistration function prototype called before the replica catalog
 * information is updated.
 *
 * This function @b must @b not
 * 	- modify the file that has been published or copied
 * 	- modify information in the replica catalog
 *
 * This function @b must
 * 	- be thread safe
 * 	- be able to be called multiple times (for example, during
 * 	  multiple restart attempts)
 *
 * No attempt is made to roll back any effects of this function.
 *
 * @ingroup globus_replica_management_session
 *
 * @param user_arg
 *        an opaque user-supplied argument to the function
 * @param destination_url
 *        the URL of the destination file
 *
 * @return may be @c GLOBUS_SUCCES or may point to an error object
 */
typedef globus_result_t
(*globus_replica_management_preregister_function_t)(
    void *                              user_arg,
    char *                              destination_url);

/**
 * Publish a file into a replica catalog collection and location and
 * copy the file from its source into the replica catalog location's
 * storage system.
 *
 * Throughout, the replica catalog remains consistent: any files
 * listed in a replica catalog location are complete and uncorrupt
 * (barring spontaneous corruption by a storage system).  Pending
 * operations on files are locked and their state is checkpointed to the
 * replica catalog so that, in the event of failure, rollback and
 * restart information is available.
 *
 * The caller must have authorization to add a filename to the replica
 * catalog collection and location, to write the file to the storage
 * system, and to read the source file using their own credentials.
 *
 * @ingroup globus_replica_management_creation
 *
 * @param handle 
 * 	  the session handle
 * @param source_url
 * 	  the URL of the file to be published into the replica catalog
 * 	  and copied to the replica catalog location's storage system,
 * 	  the URL can have the protocol schemes @c file: , @c ftp: ,
 * 	  @c gsiftp: .
 * @param source_attr
 *        GridFTP client attributes associated with the source URL, may
 *        be GLOBUS_NULL, in which case default attributes are assumed
 * @param dest_collection_url
 * 	  the LDAP URL of an existing replica catalog collection to add
 * 	  the file to
 * @param dest_collection_attrs
 *        replica catalog collection attributes. This parameter may be
 *        GLOBUS_NULL, in which case default attributes are assumed
 * @param dest_location_name
 * 	  the name of an existing replica catalog location in the
 * 	  replica catalog collection to add the file to
 * @param dest_location_attrs
 *        GridFTP client attributes associated with the destination
 *        location, may be GLOBUS_NULL, in which case default attributes
 *        are assumed
 * @param dest_filename
 * 	  the logical file name of the file to add
 * @param behavior
 * 	  the overwrite behavior if the file already exists on the
 * 	  destination storage system, if an error occurs when
 * 	  specifying:
 * 	  <ul>
 * 	  <li> #GLOBUS_REPLICA_MANAGEMENT_OVERWRITE_DISALLOW then neither
 * 	       the file nor the replica catalog are modified;
 * 	  <li> #GLOBUS_REPLICA_MANAGEMENT_OVERWRITE_SAFE then the original
 * 	       file is left intact (and is available via the replica
 * 	       catalog throughout the transfer); or
 * 	  <li> #GLOBUS_REPLICA_MANAGEMENT_OVERWRITE_UNSAFE then the original
 * 	       file is corrupted and a rollback would attempt to remove
 * 	       the original file (which is unavailable via the replica
 * 	       catalog throughout the copying operation)
 * 	  </ul>
 * @param preregister
 *        a user-supplied function called between copying the file and
 *        registering the file in the replica catalog, the function may
 *        not alter the file or the replica catalog
 * @param preregister_arg
 *        a user-supplied argument to the preregister function
 */
globus_result_t
globus_replica_management_file_publish(
    globus_replica_management_handle_t *              handle,
    char *                                            source_url,
    globus_ftp_client_operationattr_t *               source_attrs,
    char *                                            dest_collection_url,
    globus_replica_catalog_collection_handleattr_t *  dest_collection_attrs,
    char *                                            dest_location_name,
    globus_ftp_client_operationattr_t *               dest_location_attrs,
    char *                                            dest_filename,
    globus_replica_management_overwrite_behavior_t    overwrite_behavior,
    globus_replica_management_preregister_function_t  preregister,
    void *                                            preregister_arg);

/**
 * @defgroup globus_replica_management_file File Maintenance
 *
 * Maintain replica catalog locations and the associated file on
 * particular storage systems.
 */

/**
 * Copy a file from one replica catalog location's storage system to
 * another and update the replica catalog collection and location
 * information.
 *
 * Throughout, the replica catalog remains consistent: any files
 * listed in a replica catalog location are complete and uncorrupt
 * (barring spontaneous corruption by a storage system).  Pending
 * operations on files are locked and their state is checkpointed to the
 * replica catalog so that, in the event of failure, rollback and
 * restart information is available.
 *
 * The caller must have authorization to add a filename to the replica
 * catalog collection and location, to write the file to the destination
 * storage system, and to read from the source storage system.
 *
 * @ingroup globus_replica_management_file
 *
 * @param handle 
 * 	  the session handle
 * @param collection_url
 * 	  the LDAP URL of an existing replica catalog collection to copy
 * 	  the file in
 * @param collection_attrs
 *        the replica catalog collection attributes. This parameter may be
 *        GLOBUS_NULL, in which case default attributes are assumed
 * @param source_location_name
 * 	  the name of an existing replica catalog location in the
 * 	  replica catalog collection to copy the file from
 * @param source_location_attrs
 *        the GridFTP client attributes associated with the source
 *        location, may be GLOBUS_NULL, in which case default attributes
 *        are assumed
 * @param dest_location_name
 * 	  the name of an existing replica catalog location in the
 * 	  replica catalog collection to add the file to
 * @param dest_location_attrs
 *        the GridFTP client attributes associated with the destination
 *        location, may be GLOBUS_NULL, in which case default attributes
 *        are assumed
 * @param filename
 * 	  the filename of the file to add, which must exist in the
 * 	  storage system directory referred to by the replica catalog
 * 	  locations's URL constructor and must match the file
 * 	  specification for this replica catalog location
 * @param behavior
 * 	  the overwrite behavior if the file already exists on the
 * 	  destination storage system, if an error occurs when
 * 	  specifying:
 * 	  <ul>
 * 	  <li> #GLOBUS_REPLICA_MANAGEMENT_OVERWRITE_DISALLOW then neither
 * 	       the file nor the replica catalog are modified;
 * 	  <li> #GLOBUS_REPLICA_MANAGEMENT_OVERWRITE_SAFE then the original
 * 	       file is left intact (and is available via the replica
 * 	       catalog throughout the transfer); or
 * 	  <li> #GLOBUS_REPLICA_MANAGEMENT_OVERWRITE_UNSAFE then the original
 * 	       file is corrupted and a rollback would attempt to remove
 * 	       the original file (which is unavailable via the replica
 * 	       catalog throughout the copying operation)
 * 	  </ul>
 * @param preregister
 *        a user-supplied function called between copying the file and
 *        registering the file in the replica catalog, may not alter the
 *        file or the replica catalog
 * @param preregister_arg
 *        a user-supplied argument to the preregister function
 */
globus_result_t
globus_replica_management_file_copy(
    globus_replica_management_handle_t *              handle,
    char *                                            collection_url,
    globus_replica_catalog_collection_handleattr_t *  collection_attrs,
    char *                                            source_location_name,
    globus_ftp_client_operationattr_t *               source_location_attrs,
    char *                                            dest_location_name,
    globus_ftp_client_operationattr_t *               dest_location_attrs,
    char *                                            filename,
    globus_replica_management_overwrite_behavior_t    overwrite_behavior,
    globus_replica_management_preregister_function_t  preregister,
    void *                                            preregister_arg);

/**
 * Remove a filename from the replica catalog location and
 * the file from the replica catalog location's storage system.
 *
 * If @c delete_file is not @c GLOBUS_TRUE then don't also delete the
 * file from the replica catalog location's storage system.
 *
 * The caller must have authorization to remove a filename from the
 * replica catalog location and to delete the file from the source
 * storage system.
 *
 * @ingroup globus_replica_management_file
 *
 * @param handle 
 * 	  the session handle
 * @param collection_url
 * 	  the LDAP URL of an existing replica catalog collection to
 * 	  remove the file from
 * @param collection_attrs
 *        the replica catalog collection attributes, may be GLOBUS_NULL,
 *        in which case default attributes are assumed
 * @param location_name
 * 	  the name of an existing replica catalog location in the
 * 	  replica catalog collection to remove the file from
 * @param location_attrs
 *        the GridFTP client attributes associated with the location,
 *        may be GLOBUS_NULL, in which case default attributes are
 *        assumed
 * @param filename
 * 	  the filename of the file to remove, which must exist in the
 * 	  storage system directory referred to by the replica catalog
 * 	  locations's URL constructor
 * @param delete_file
 * 	  whether to delete the file from the replica catalog location's
 * 	  storage system
 */
globus_result_t
globus_replica_management_file_delete(
    globus_replica_management_handle_t *              handle,
    char *                                            collection_url,
    globus_replica_catalog_collection_handleattr_t *  collection_attrs,
    char *                                            location_name,
    globus_ftp_client_operationattr_t *               location_attrs,
    char *                                            filename,
    globus_bool_t                                     delete_file);

/**
 * Restart the operation described by the given lock.
 *
 * @ingroup globus_replica_management_session
 *
 * @param handle 
 * 	  the session handle
 * @param rollback_info
 *        an opaque contact string containing a pointer to rollback and
 *        restart information for the operation to be restarted
 * @param collection_attrs
 *        the replica catalog collection attributes, may be GLOBUS_NULL, in
 *        which case default attributes are assumed
 * @param source_ftp_attrs
 *        the GridFTP client attributes associated with the source URL,
 *        may be GLOBUS_NULL, in which case default attributes are
 *        assumed 
 * @param destination_ftp_attrs
 *        the GridFTP client attributes associated with the destination
 *        URL, may be GLOBUS_NULL, in which case default attributes are
 *        assumed
 * @param preregister
 *        a user-supplied function called between copying the file and
 *        registering the file in the replica catalog, the function must
 *        not alter the file or the replica catalog and must be able to
 *        be called multiple times as there may be multiple restart
 *        attempts
 * @param preregister_arg
 *        a user-supplied argument to the preregister function
 */
globus_result_t
globus_replica_management_restart(
    globus_replica_management_handle_t *              handle,
    char *                                            rollback_info,
    globus_replica_catalog_collection_handleattr_t *  collection_attrs,
    globus_ftp_client_operationattr_t *               source_ftp_attrs,
    globus_ftp_client_operationattr_t *               destination_ftp_attrs,
    globus_replica_management_preregister_function_t  preregister,
    void *                                            preregister_arg);

/**
 * Rollback the operation described by the given lock.
 *
 * @ingroup globus_replica_management_session
 *
 * @param handle 
 * 	  the session handle
 * @param rollback_info
 *        an opaque string containing a contact string for rollback and
 *        restart information for an operation to be rolled back
 * @param collection_attrs
 *        the replica catalog collection attributes, may be GLOBUS_NULL,
 *        in which case default attributes are assumed
 * @param ftp_attrs
 *        the GridFTP client attributes associated with the destination
 *        URL, may be GLOBUS_NULL, in which case default attributes are
 *        assumed
 */
globus_result_t
globus_replica_management_rollback(
    globus_replica_management_handle_t *                handle,
    char *                                              rollback_info,
    globus_replica_catalog_collection_handleattr_t *    collection_attrs,
    globus_ftp_client_operationattr_t *                 ftp_attrs);

/**
 * Obtain the rollback and restart information contact string from the
 * result of a replica management operation.
 *
 * The operations that support rollback or restart are
 * globus_replica_management_file_publish(),
 * globus_replica_management_file_copy(), and
 * globus_replica_management_file_delete().
 *
 * @ingroup globus_replica_management_session
 *
 * @param result
 *        the result of a failed operation which allows rollback or
 *        restart
 * @param contact_string
 *        the rollback and restart information contact string associated
 *        with the operation, the callee must free this string
 */
globus_result_t
globus_replica_management_get_rollback_contact(
    globus_result_t                     result,
    char **                             contact_string);

#ifdef EXTERN_C_END
EXTERN_C_END
#endif

#endif /* _GLOBUS_REPLICA_MANAGEMENT_H */
